#include <fcntl.h>

#include "gamesdef.h"
#include "misc.h"
#include "bigmap.h"

int bigmapf;
char bigmap[WORLDSIZE];
char bigmapfil[] = EMPPATH(data/bigmap);

/* ________________________________________________________________
**
** bigmap file utilities
** ________________________________________________________________
*/

openbigmap(cnum)
int cnum;
{
  char name[80];
  int i;
  sprintf(name,"%s%02d",bigmapfil,cnum);
  bigmapf=open(name, O_CREAT | O_RDWR, 0600);
  for (i=0;i<WORLDSIZE;i++) bigmap[i]=' ';
  if (bigmapf>=0) read(bigmapf, bigmap, WORLDSIZE);
}

closebigmap()
{
  if (bigmapf>=0) close(bigmapf);
  bigmapf = -1;
}

writebigmap()
{
  if (bigmapf>=0) {
    lseek(bigmapf, 0L, 0);
    write(bigmapf, bigmap, WORLDSIZE);
  }
}

setbigmap(x,y,new)
int x,y;
unsigned char new;
{
    /*
    **  A 'new' designation of '?' is only written to the
    **  database if the old designation is ocean, wilderness,
    **  mountain, or unknown.  This prevents more informative
    **  sector designations from being overwritten by '?'.
    **
    **  There are also several "special" new designations
    **  that affect a sector's flag bit (bit 7) without
    **  affecting the sector's designation.
    **
    ** 		0x80 - clears the flag bit
    **  	0x81 - set the flag bit
    **  	0x82 - toggle the flag bit
    */

    unsigned char old,oldflag;
    x  = normalize(x,WORLD_X);
    y  = normalize(y,WORLD_Y);
    if (valid(x,y)) {

	old      = stype(x,y);
	oldflag  = old & 0x80;
	old     &= 0x7F;

	switch (new) {
	case 0x80:
	    stype(x,y) = old;
	    break;
	case 0x81:
	    stype(x,y) = old | 0x80;
	    break;
	case 0x82:
	    stype(x,y) = old | (oldflag ^ 0x80);
	    break;
	case '?':
	    if (old!='.' && old!='-' && old!=' ' && old!='^') break;
	default:
	    new &= 0x7F;
	    stype(x,y) = new | oldflag;
	}
    }
}

unsigned char getbigmap(x,y)
int x,y;
{
  x  = normalize(x,WORLD_X);
  y  = normalize(y,WORLD_Y);
  return(valid(x,y) ? stype(x,y) : (char)0 );
}
