/*
**  Lotto routines
**
**  6/91 Bill Frolik
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "gamesdef.h"
#include "misc.h"
#include "file.h"
#include "lotto.h"

char *lockfil1 = EMPPATH(data/lotto.lock1);
char *lockfil2 = EMPPATH(data/lotto.lock2);
char *lottofil = EMPPATH(data/lotto);
int lottof = -1;

closelotto()
{
    if (lottof >= 0) {
	close(lottof);
	lottof = -1;
    }
}

int openlotto()
{
    union lottorec lottery;
    closelotto();
    lottof = open(lottofil, O_CREAT | O_RDWR, 0600);
    if (lottof >= 0) {
	lseek(lottof,0L,0);
	lottery.jackpot.primary = 0;
	lottery.jackpot.secondary = 0;
	if (read(lottof,&lottery.ticket,sizeof(tkt)) != sizeof(tkt)) {
	    lseek(lottof,0L,0);
	    lottery.jackpot.primary = LOTTOSEED;
	    lottery.jackpot.secondary = 0;
	    write(lottof,&lottery.ticket,sizeof(tkt));
	}
    }
    return (lottof>=0 ? lottery.jackpot.primary : 0);
}

locklotto(block,n)
int block,n;
{
    int i;
    struct stat statbuf;
    if (block) {
	close(creat(lockfil2,0600));
	if (stat(lockfil2,&statbuf)) 
	    return(0);
    } else {
	if (!stat(lockfil2,&statbuf)) 
	    return(0);
    }
    for (i=n; i && !stat(lockfil1, &statbuf); i--) {
	pr("Waiting on lock ...\n");
	sleep(roll(3));
    }
    if (i) {
	close(creat(lockfil1,0600));
	i = !stat(lockfil1,&statbuf);
    }
    if (!i)
	pr("Lottery file busy, try again later.\n");
    return(i);
}

unlocklotto()
{
    unlink(lockfil2);
    unlink(lockfil1);
}

resetlotto()
{
    unlink(lottofil);
    openlotto();
    closelotto();
}

int readticket(ticket)
tkt *ticket;
{
    if (lottof<0) return(0);
    return (read(lottof,ticket,sizeof(tkt))==sizeof(tkt));
}

int readjackpot(ticket)
tkt *ticket;
{
    if (lottof<0) return(0);
    lseek(lottof,0L,0);
    return (readticket(ticket));
}

setjackpot(add,amt1,amt2)
int add,amt1,amt2;
{
    int i;
    union lottorec lottery;
    if (openlotto()) {
	i = readjackpot(&lottery.ticket);
	if (i && add) {
	    if (amt1>0) lottery.jackpot.primary += amt1;
	    if (amt2>0) lottery.jackpot.secondary += amt2;
	} else {
	    if (amt1>=0) lottery.jackpot.primary = amt1;
	    if (amt2>=0) lottery.jackpot.secondary = amt2;
	}
	lseek(lottof,0L,0);
	write(lottof,&lottery.ticket,sizeof(tkt));
	closelotto();
    }
}

addjackpot(amt1,amt2)
int amt1,amt2;
{
    setjackpot(1,amt1,amt2);
}

addticket(ticket)
tkt *ticket;
{
    if (openlotto()) {
	lseek(lottof,0L,2);
	if (write(lottof,ticket,sizeof(tkt)) == sizeof(tkt)) {
	    addjackpot(0,ticket->cost);
	    return(1);
	}
	closelotto();
    }
    return(0);
}

drawlotto(primary,secondary,picks,windfall)
int *primary,*secondary,*picks,*windfall;
{
    float winamount,fjackpot;
    int h,i,j,k;
    int ball[LOTTOBALLS];
    int splitways, portion[MAXNOC], numwinners;
    union lottorec lottery;

    for (i=0; i<LOTTOBALLS; i++) ball[i]=i+1;
    for (h=roll(LOTTOBALLS*2); h; h--) 
	for (i=0; i<LOTTOBALLS; i++) {
	    j = roll(LOTTOBALLS-1);
	    k=ball[i]; ball[i]=ball[j]; ball[j]=k;
	}
    for (i=0; i<NUMPICKS-1; i++)
	for (j=i+1; j<NUMPICKS; j++)
	    if (ball[i]>ball[j]) {
		k=ball[i]; ball[i]=ball[j]; ball[j]=k;
	    }
    for (i=0; i<NUMPICKS; i++) picks[i] = ball[i];

    if (openlotto()) {
	readjackpot(&lottery.ticket);
	*primary = lottery.jackpot.primary;
	*secondary = lottery.jackpot.secondary;
	splitways = 0;
	for (i=0; i<MAXNOC; i++) portion[i]=0;
	while (readticket(&lottery.ticket)) {
	    for (i=j=0; i<NUMPICKS; i++)
		if (lottery.ticket.pick[i]==ball[i]) j++;
	    if (j==NUMPICKS) {
		splitways += lottery.ticket.cost;
		portion[lottery.ticket.owner] += lottery.ticket.cost;
	    }
	}
	for (i=j=0; i<MAXNOC; i++) {
	    windfall[i] = 0;
	    if (portion[i]) {
		windfall[i]=(int)
		    ((float)(*primary)*(float)portion[i]/(float)splitways);
		j++;
	    }
	}
	closelotto();
    }
    return (j);
}

clearlotto(restart) /* prepare lotto for next round */
int restart;
{
    int n;
    union lottorec lottery,temp;
    lottery.jackpot.primary = LOTTOSEED;
    lottery.jackpot.secondary = 0;
    if (openlotto()) {
	readjackpot(&lottery.ticket); /* get current jackpot */
	for (n=0; readticket(&temp.ticket); n++); /* count tickets */
    }
    closelotto();
    resetlotto(); /* reset ticket list, reset jackpots */
    if (!restart) /* no winners? if >10 tickets were sold, bump jackpots */
	if (n >= 10) lottery.jackpot.primary += (LOTTOSEED>>1);
	setjackpot(0,(int)lottery.jackpot.primary,
		   (int)(lottery.jackpot.secondary>>2));
}
