/*
**  Misc ship database routines
**
**  6/91 - Bill Frolik
*/

#include "misc.h"
#include "ship.h"

hassonar(s)
struct shpstr *s;    
{
    int i;
    if (isballoon(s) && s->shp_depth>1) return 0;
    return(mchr[s->shp_type].m_flags & M_SONAR);
}

issub(s)
struct shpstr *s;    
{ return(mchr[s->shp_type].m_flags & M_SUB); }

vehicleclass(s)
struct shpstr *s;    
{ return(mchr[s->shp_type].m_class); }

isship(s) 
struct shpstr *s;    
{ return(mchr[s->shp_type].m_class == CLASS_SHIP); }

istank(s) 
struct shpstr *s;    
{ return(mchr[s->shp_type].m_class == CLASS_TANK); }

isballoon(s) 
struct shpstr *s;    
{ return(mchr[s->shp_type].m_class == CLASS_BALLOON); }

istrain(s)
struct shpstr *s;    
{ return(mchr[s->shp_type].m_class == CLASS_TRAIN); }

#ifdef BIMOTOR
/* ___________________________________________________________
**|                                                           |
**| whichengine(sp,fueltype,fuelamt)                          |
**|                                                           |
**| Determines which engine (0 or 1) to use on the specified  |
**| ship.  Also returns the type of fuel required and amount  |
**| of fuel currently aboard.                                 |
**|                                                           |
**| If the ship doesn't need an engine, return -1.	      |
**| If the ship is out of fuel, return 0 with fuelamt=0.      |
**|___________________________________________________________|
*/

int whichengine(s,fueltype,fuelamt)	/* return engine number (or -1) */
struct shpstr *s;			/* ship pointer */
u_char *fueltype;			/* return type of fuel required */
int *fuelamt;				/* return amt of fuel available */
{
    struct mchrstr *mcp;
    int i;

    *fueltype = 0;
    *fuelamt = 0;

    mcp = &mchr[s->shp_type];
    if (!mcp->m_engine[0].fuel) return -1; /* needs no engine */

    for (i=(issub(s) && (s->shp_depth>0)) ? 1 : 0; i < 2; i++) {
	*fueltype = mcp->m_engine[i].fuel;
	*fuelamt = *fueltype ? getvar(*fueltype, (char *)s, EF_SHIP) : 0;
	if (*fuelamt) break;
    }

    return (i & 1);
}

/* _______________________________________________________________________
**|                                                                       |
**| runengine(s,rate)                                                     |
**|                                                                       |
**| Run the appropriate engine for ship "s".  This is done by figuring    |
**| out which engine to run, bumping the odometer, and consuming fuel.    |
**| We also recharge the other engine's fuel supply a bit if necessary.   |
**| Fuel is consumed at the specified rate (1=normal rate, 2=2x normal,   |
**| 3=3x normal, and so forth).                                           |
**|                                                                       |
**| Returns 0 if ship/vehicle shouldn't be able to move.                  |
**| Note that this routine doesn't do a final "putship".		  |
**|_______________________________________________________________________|
*/

int runengine(s,rate)
struct shpstr *s;
int rate;
{
    struct mchrstr *mcp;
    estr *engspec;
    u_char fueltype,*odom;
    int i,j,k,maxamt,fuelamt,*dspu;
    double factor,odoincr;

    mcp = &mchr[s->shp_type];
    engine = whichengine(s,&fueltype,&fuelamt);
    if (engine<0) return 1; /* has no engine, just pretend engine ran */
    if (fuelamt==0) return 0; /* out of fuel */
	
    engspec  = &(mcp->m_engine[engine]);
    odom     = &(s->shp_odom[engine]);

    /*
    **  Compute engine efficiency factor.
    **  This will be large (1.0 to 100.0) for efficient, high-tech 
    **  ships; small (0.01 to 1.0) for inefficient, low-tech ships.
    */

    factor  = (double)(shpp->shp_tech+200)/200.0;
    factor *= (double)(shpp->shp_effic)/100.0;
    if (factor < 0.01) factor = 0.01;
    if (factor > 100.0) factor = 100.0;

    /* 
    **  Consume fuel in the selected engine:
    **  1. Bump the decisector odometer by nominally 10 (less for
    **     more efficient ships, more for less efficient ships).
    **  2. If odometer is greater than "dspu", decrement fuel by
    **     one unit, subtract "dspu" from odometer, and repeat
    **     step 2.
    */

    odoincr  = 10.0/factor;
    if (odoincr < 1.0) odoincr = 1.0; /* guarantee bump by 1 */
    k = (int)(*odom) + (int)(odoincr+0.5)*rate;
    for (i=fuelamt; (k > engspec->dspu) && (i>0); i--)
	k -= engspec->dspu;
    *odom = k;
    if (i != fuelamt) 
	putvar(fueltype, i, (char *)s, EF_SHIP);
	
    /* 
    **  Recharge fuel in the other system if necessary.
    **  This is done by turning back the other engine's odometer
    **  by some small amount, and increasing it's fuel reserve
    **  if it went below zero.
    */
    
    i = (int)engspec->dspsm; /* amt by which to turn it back */

    if (i > 0) {
	engine   = 1-engine;
	engspec  = &(mcp->m_engine[engine]);
	odom     = &(s->shp_odom[engine]);
	fueltype = mcp->m_engine[engine].fuel;
	fuelamt  = fueltype ? getvar(fueltype, (char *)s, EF_SHIP) : 0;

	*odom -= i;
	for (j=fuelamt; *odom < 0; j++) 
	    *odom += engspec->dspu;
	maxamt = vl_find(fueltype, vbase->m_vtype,
			 vbase->m_vamt, (int)vbase-m_nv);
	if (j > maxamt) 
	    j = maxamt;
	if (fuelamt != j) 
	    putvar(fueltype, j, (char *)s, EF_SHIP);
    }
    
    return 1;
}
#endif
