static char *RCSid = "$Header: /sequent2/empire/EMP/lib/global/RCS/ship.c,v 1.2 89/09/10 21:37:57 mr-frog Exp $";

/*
** ship.c
**
** marine/tank/airship characteristics
*/

#include "misc.h"
#include "var.h"
#include "ship.h"

#ifdef SHIPSNEEDPET
#define	mpg(x)	x
#else
#define mpg(x)	0
#endif

/*
** For vessels and vehicles that burn petrol, the MPG rating determines 
** the rate at which petrol is burned according to the following formula:
**
**                          (MPGrating * (tech+200) * efficiency)
**      sectors/petrol  =   -------------------------------------
**   			                   200,000
*/

struct  mchrstr mchr[]  = {

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   25, 15,
  /* armor & speed ratings         */   10, 20,
  /* visibility & viewing range    */   15, 2,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "fishing boat",
  /* tech & cost to build          */   0, 200,
  /* special features              */   M_FOOD,
  /* mpg rating                    */   0,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   4,
  { V_CIVIL, V_MILIT, V_FOOD, V_UW },
  {      30,       5,    100,   15 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   80, 30,
  /* armor & speed ratings         */   20, 40,
  /* visibility & viewing range    */   20, 10,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "fishing trawler",
  /* tech & cost to build          */   180, 6000,
  /* special features              */   M_FOOD|M_SONAR|M_SPY,
  /* mpg rating                    */   mpg(45),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   4,
  { V_CIVIL, V_MILIT, V_FOOD, V_PETROL },
  {      25,      10,    120,       30 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   60, 40,
  /* armor & speed ratings         */   50, 25,
  /* visibility & viewing range    */   35, 3,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "cargo ship",
  /* tech & cost to build          */   0, 750,
  /* special features              */   M_TAC,
  /* mpg rating                    */   mpg(50),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   8, 2,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   9,
  { V_CIVIL, V_MILIT, V_SHELL, V_GUN, V_FOOD, V_LCM, V_HCM, V_UW, V_PETROL },
  {      50,      50,     300,    50,    900,  1400,   900,  250,       50 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   40, 40,
  /* armor & speed ratings         */   20, 70,
  /* visibility & viewing range    */   17, 5,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "yacht",
  /* tech & cost to build          */   220, 2500,
  /* special features              */   M_LAND,
  /* mpg rating                    */   0,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   4,
  { V_CIVIL, V_MILIT, V_FOOD, V_UW }, 
  {      60,      30,     20,   30 }},
  
/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   40, 40,
  /* armor & speed ratings         */   80, 25,
  /* visibility & viewing range    */   60, 3,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "tanker",
  /* tech & cost to build          */   50, 1500,
  /* special features              */   0,
  /* mpg rating                    */   mpg(50),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   6,
  { V_CIVIL, V_MILIT, V_PETROL, V_FOOD, V_OIL, V_UW },
  {      30,       5,      250,    200,   250,   25}},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   50, 50,
  /* armor & speed ratings         */   127, 10,
  /* visibility & viewing range    */   80, 4,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "oil derrick",
  /* tech & cost to build          */   100, 2000,
  /* special features              */   M_OIL|M_HELI|M_FOOD,
  /* mpg rating                    */   mpg(30),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 1,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   6,
  { V_CIVIL, V_MILIT, V_FOOD, V_OIL, V_UW, V_PETROL },
  {      80,      80,    300,   200,  180,      100 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   20, 10,
  /* armor & speed ratings         */   30, 50,
  /* visibility & viewing range    */   10, 4,
  /* firing range, number of guns  */   1, 1,
  /* ship name                     */   "patrol boat",
  /* tech & cost to build          */   40, 350,
  /* special features              */   0,
  /* mpg rating                    */   mpg(150),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_SHELL, V_GUN, V_FOOD, V_PETROL }, 
  {      10,      10,     2,     30,       10 }},
  
/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   25, 25,
  /* armor & speed ratings         */   60, 30,
  /* visibility & viewing range    */   20, 3,
  /* firing range, number of guns  */   1, 1,
  /* ship name                     */   "minesweep",
  /* tech & cost to build          */   20, 550,
  /* special features              */   M_SWEEP,
  /* mpg rating                    */   mpg(60),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_SHELL, V_GUN, V_FOOD, V_PETROL },
  {      25,      60,     2,     25,       20 }},
  
/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   20, 30,
  /* armor & speed ratings         */   70, 30,
  /* visibility & viewing range    */   15, 3,
  /* firing range, number of guns  */   2, 2,
  /* ship name                     */   "guided missile frigate",
  /* tech & cost to build          */   130, 2000,
  /* special features              */   M_MSL,
  /* mpg rating                    */   mpg(60),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 15,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   6,
  { V_MILIT, V_SHELL, V_GUN, V_PETROL, V_FOOD, V_PETROL }, 
  {      50,      30,     3,      100,     70,       25 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   25, 35,
  /* armor & speed ratings         */   80, 35,
  /* visibility & viewing range    */   20, 4,
  /* firing range, number of guns  */   3, 2,
  /* ship name                     */   "destroyer",
  /* tech & cost to build          */   20, 600,
  /* special features              */   M_MINE|M_SONAR|M_DCH|M_MSL,
  /* mpg rating                    */   mpg(60),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 2,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_SHELL, V_GUN, V_FOOD, V_PETROL },
  {      80,      40,     4,     80,       20 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   40, 40,
  /* armor & speed ratings         */   30, 30,
  /* visibility & viewing range    */   30, 2,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "landing ship",
  /* tech & cost to build          */   70, 1200,
  /* special features              */   M_LAND,
  /* mpg rating                    */   mpg(40),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   4, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   3,
  { V_MILIT, V_FOOD, V_PETROL },
  {     400,    300,       10 }},
  
/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   40, 50,
  /* armor & speed ratings         */   100, 30,
  /* visibility & viewing range    */   30, 5,
  /* firing range, number of guns  */   6, 3,
  /* ship name                     */   "heavy cruiser",
  /* tech & cost to build          */   40, 3500,
  /* special features              */   M_MINE|M_MSL,
  /* mpg rating                    */   mpg(40),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 3,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_SHELL, V_GUN, V_FOOD, V_PETROL }, 
  {     120,      60,     6,    100,       30 }},
  
/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   55, 65,
  /* armor & speed ratings         */   127, 30,
  /* visibility & viewing range    */   35, 6,
  /* firing range, number of guns  */   8, 4,
  /* ship name                     */   "battleship",
  /* tech & cost to build          */   50, 5000,
  /* special features              */   M_MSL,
  /* mpg rating                    */   mpg(40),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 4,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_SHELL, V_GUN, V_FOOD, V_PETROL },
  {     200,     100,     8,    300,      70 }},
  
/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   70, 65,
  /* armor & speed ratings         */   80, 30,
  /* visibility & viewing range    */   35, 8,
  /* firing range, number of guns  */   2, 2,
  /* ship name                     */   "aircraft carrier",
  /* tech & cost to build          */   60, 7500,
  /* special features              */   M_FLY|M_TAC,
  /* mpg rating                    */   mpg(40),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   2, 12,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_SHELL, V_GUN, V_PETROL, V_FOOD },
  {     350,     350,     4,      500,    180}},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   40, 30,
  /* armor & speed ratings         */   30, 25,
  /* visibility & viewing range    */   12, 3,
  /* firing range, number of guns  */   2, 2,
  /* ship name                     */   "submarine",
  /* tech & cost to build          */   70, 2200,
  /* special features              */   M_SUB|M_TORP|M_MINE|M_SONAR,
  /* mpg rating                    */   mpg(80),
  /* max air (submarines only)     */	3000,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_SHELL, V_GUN, V_FOOD, V_PETROL },
  {      25,      25,     4,     80,       20 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   90, 60,
  /* armor & speed ratings         */   30, 50,
  /* visibility & viewing range    */   10, 3,
  /* firing range, number of guns  */   4, 4,
  /* ship name                     */   "nuclear sub",
  /* tech & cost to build          */   320, 4000,
  /* special features              */   M_SUB|M_TORP|M_SONAR|M_MSL,
  /* mpg rating                    */   0,
  /* max air (submarines only)     */	15000,
  /* max tanks, planes             */   0, 2,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   7,
  { V_MILIT, V_SHELL, V_GUN, V_PETROL, V_FOOD, V_RAD },
  {      30,      24,     4,       60,    500,    10 }},      

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   60, 40,
  /* armor & speed ratings         */   20, 25,
  /* visibility & viewing range    */   25, 3,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "slave ship",
  /* tech & cost to build          */   8, 750,
  /* special features              */   0,
  /* mpg rating                    */   0,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_CIVIL, V_MILIT, V_FOOD, V_UW, V_BAR },
  {      20,      80,    200, 1200,    80 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */	40, 40,
  /* armor & speed ratings         */	40, 25,
  /* visibility & viewing range    */	50, 3,
  /* firing range, number of guns  */	0, 0,
  /* ship name                     */	"ore ship",
  /* tech & cost to build          */	25, 750,
  /* special features              */	0,
  /* mpg rating                    */   mpg(40),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */	9,
  { V_CIVIL, V_MILIT, V_IRON, V_DUST, V_FOOD, V_UW, V_RAD, V_PETROL, V_BAR },
  {      30,       5,    990,    990,    200,   45,   990,       40,   200 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   80, 40,
  /* armor & speed ratings         */   5, 30,
  /* visibility & viewing range    */   30, 3,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "cruise ship",
  /* tech & cost to build          */   60, 3000,
  /* special features              */   0,
  /* mpg rating                    */   mpg(60),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   4,
  { V_CIVIL, V_MILIT, V_FOOD, V_PETROL },
  {     300,       5,    300,       40 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_BALLOON,
  /* lcms & hcms to build          */	10, 2,
  /* armor & speed ratings         */	1, 20,
  /* visibility & viewing range    */	45, 2,
  /* firing range, number of guns  */	0, 0,
  /* ship name                     */	"balloon",
  /* tech & cost to build          */	30, 150,
  /* special features              */	0,
  /* mpg rating                    */   0,
  /* max air (envelope volume)     */	1000,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   balloonterrain,
  /* num items in cargo list       */	3,
  { V_CIVIL, V_MILIT, V_FOOD },
  {       1,       1,     10 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_BALLOON,
  /* lcms & hcms to build          */	25, 5,
  /* armor & speed ratings         */	1, 25,
  /* visibility & viewing range    */	45, 3,
  /* firing range, number of guns  */	0, 0,
  /* ship name                     */	"blimp",
  /* tech & cost to build          */	60, 400,
  /* special features              */	0,
  /* mpg rating                    */   0,
  /* max air (envelope volume)     */	2500,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   balloonterrain,
  /* num items in cargo list       */	4,
  { V_CIVIL, V_MILIT, V_FOOD, V_UW },
  {       3,       3,     30,    3 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_BALLOON,
  /* lcms & hcms to build          */	50, 10,
  /* armor & speed ratings         */	5, 25,
  /* visibility & viewing range    */	80, 5,
  /* firing range, number of guns  */	1, 1,
  /* ship name                     */	"airship",
  /* tech & cost to build          */	100, 1800,
  /* special features              */	M_SPY,
  /* mpg rating                    */   0,
  /* max air (envelope volume)     */	5000,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   balloonterrain,
  /* num items in cargo list       */	5,
  { V_MILIT, V_FOOD, V_UW, V_SHELL, V_GUN },
  {      20,     50,   10,      20,     1 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TANK,
  /* lcms & hcms to build          */	15, 25,
  /* armor & speed ratings         */	10, 40,
  /* visibility & viewing range    */	3, 1,
  /* firing range, number of guns  */	4, 1,
  /* ship name                     */	"tank 1",
  /* tech & cost to build          */	20, 200,
  /* special features              */	0,
  /* mpg rating                    */   30,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   tankterrain,
  /* num items in cargo list       */	5,
  { V_MILIT, V_FOOD, V_SHELL, V_GUN, V_PETROL },
  {       4,     20,      20,     1,       10 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TANK,
  /* lcms & hcms to build          */	15, 35,
  /* armor & speed ratings         */	60, 60,
  /* visibility & viewing range    */	3, 1,
  /* firing range, number of guns  */	6, 1,
  /* ship name                     */	"tank 2",
  /* tech & cost to build          */	50, 400,
  /* special features              */	0,
  /* mpg rating                    */   35,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   tankterrain,
  /* num items in cargo list       */	5,
  { V_MILIT, V_FOOD, V_SHELL, V_GUN, V_PETROL },
  {       6,     40,      30,     2,       10}},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TANK,
  /* lcms & hcms to build          */	20, 30,
  /* armor & speed ratings         */	20, 60,
  /* visibility & viewing range    */	3, 1,
  /* firing range, number of guns  */	0, 0,
  /* ship name                     */	"armored carrier",
  /* tech & cost to build          */	50, 170,
  /* special features              */	0,
  /* mpg rating                    */   30,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   tankterrain,
  /* num items in cargo list       */	4,
  { V_MILIT, V_FOOD, V_SHELL, V_PETROL },
  {      40,     40,      40,       10}},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TANK,
  /* lcms & hcms to build          */	15, 35,
  /* armor & speed ratings         */	40, 60,
  /* visibility & viewing range    */	3, 1,
  /* firing range, number of guns  */	2, 1,
  /* ship name                     */	"assault vehicle",
  /* tech & cost to build          */	70, 800,
  /* special features              */	M_LAND,
  /* mpg rating                    */   30,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   tankterrain,
  /* num items in cargo list       */	5,
  { V_MILIT, V_FOOD, V_SHELL, V_GUN, V_PETROL },
  {      20,     40,      20,     1,       10}},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TANK,
  /* lcms & hcms to build          */	20, 40,
  /* armor & speed ratings         */	100, 60,
  /* visibility & viewing range    */	3, 1,
  /* firing range, number of guns  */	7, 2,
  /* ship name                     */	"battletank",
  /* tech & cost to build          */	100, 1800,
  /* special features              */	0,
  /* mpg rating                    */   35,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   tankterrain,
  /* num items in cargo list       */	5,
  { V_MILIT, V_FOOD, V_SHELL, V_GUN, V_PETROL },
  {       8,     40,      40,     3,       15}},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   40, 55,
  /* armor & speed ratings         */   100, 55,
  /* visibility & viewing range    */   30, 5,
  /* firing range, number of guns  */   6, 3,
  /* ship name                     */   "nuclear cruiser",
  /* tech & cost to build          */   320, 4500,
  /* special features              */   M_MSL|M_MINE|M_SONAR|M_DCH,
  /* mpg rating                    */   0,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 3,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_SHELL, V_GUN, V_FOOD, V_RAD },
  {     180,     120,     6,    200,    10 }},
  
/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   75, 85,
  /* armor & speed ratings         */   127, 60,
  /* visibility & viewing range    */   35, 9,
  /* firing range, number of guns  */   10, 6,
  /* ship name                     */   "nuclear battleship",
  /* tech & cost to build          */   340, 7000,
  /* special features              */   M_SONAR|M_MSL,
  /* mpg rating                    */   0,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 6,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_SHELL, V_GUN, V_FOOD, V_RAD },
  {     350,     400,     8,    400,    10 }},
  
/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   80, 75,
  /* armor & speed ratings         */   80, 60,
  /* visibility & viewing range    */   35, 10,
  /* firing range, number of guns  */   3, 3,
  /* ship name                     */   "nuclear carrier",
  /* tech & cost to build          */   360, 9000,
  /* special features              */   M_FLY|M_SONAR|M_TAC,
  /* mpg rating                    */   0,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   8, 40,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   6,
  { V_MILIT, V_SHELL, V_GUN, V_PETROL, V_FOOD, V_RAD },
  {     400,     450,     5,      800,    400,    10 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TANK,
  /* lcms & hcms to build          */	20, 25,
  /* armor & speed ratings         */	10, 40,
  /* visibility & viewing range    */	3, 1,
  /* firing range, number of guns  */	0, 0,
  /* ship name                     */	"rocket launcher",
  /* tech & cost to build          */	140, 1500,
  /* special features              */	M_MSL,
  /* mpg rating                    */   25,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 4,
  /* terrain restriction string    */   tankterrain,
  /* num items in cargo list       */	4,
  { V_MILIT, V_FOOD, V_SHELL, V_PETROL },
  {       4,     20,      30,       10}},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   60, 70,
  /* armor & speed ratings         */   80, 35,
  /* visibility & viewing range    */   70, 3,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "supertanker",
  /* tech & cost to build          */   150, 3000,
  /* special features              */   0,
  /* mpg rating                    */   mpg(30),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   6,
  { V_CIVIL, V_MILIT, V_PETROL, V_FOOD, V_OIL, V_UW },
  {      20,       5,      600,    100,   800,   10}},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   70, 70,
  /* armor & speed ratings         */   80, 45,
  /* visibility & viewing range    */   70, 4,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "nuclear supertanker",
  /* tech & cost to build          */   350, 10000,
  /* special features              */   0,
  /* mpg rating                    */   0,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   7,
  { V_CIVIL, V_MILIT, V_PETROL, V_FOOD, V_OIL, V_UW, V_RAD },
  {      20,       5,      990,    100,   990,   10,    10 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   35, 10,
  /* armor & speed ratings         */   10, 40,
  /* visibility & viewing range    */   17, 2,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "schooner",
  /* tech & cost to build          */   10, 500,
  /* special features              */   0,
  /* mpg rating                    */   0,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   4,
  { V_CIVIL, V_MILIT, V_FOOD, V_UW }, 
  {      10,      10,     20,    5}},
  
/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   50, 50,
  /* armor & speed ratings         */   70, 25,
  /* visibility & viewing range    */   35, 3,
  /* firing range, number of guns  */   4, 2,
  /* ship name                     */   "tank ship",
  /* tech & cost to build          */   30, 1000,
  /* special features              */   M_MSL,
  /* mpg rating                    */   mpg(30),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   20, 1,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_SHELL, V_GUN, V_FOOD,V_PETROL },
  {      20,     100,    10,     40,      80 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TRAIN,
  /* lcms & hcms to build          */   30, 50,
  /* armor & speed ratings         */   10, 55,
  /* visibility & viewing range    */   20, 1,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "passenger",
  /* tech & cost to build          */   0, 1500,
  /* special features              */   0,
  /* mpg rating                    */   mpg(80),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   trainterrain,
  /* num items in cargo list       */   5,
  { V_CIVIL, V_MILIT, V_FOOD, V_UW, V_PETROL },
  {    1400,      20,    300,  800,      200 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TRAIN,
  /* lcms & hcms to build          */   30, 50,
  /* armor & speed ratings         */   10, 55,
  /* visibility & viewing range    */   20, 1,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "freight",
  /* tech & cost to build          */   0, 1500,
  /* special features              */   M_TAC,
  /* mpg rating                    */   mpg(60),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   40, 10,
  /* terrain restriction string    */   trainterrain,
  /* num items in cargo list       */   8,
  { V_CIVIL, V_MILIT, V_FOOD, V_LCM, V_HCM, V_UW, V_OIL, V_PETROL },
  {      10,      10,   5000,  4000,  2000,   10,   800,      800 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TRAIN,
  /* lcms & hcms to build          */   30, 50,
  /* armor & speed ratings         */   10, 55,
  /* visibility & viewing range    */   20, 1,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "munitions",
  /* tech & cost to build          */   10, 1500,
  /* special features              */   M_TAC|M_MSL,
  /* mpg rating                    */   mpg(80),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   4, 4,
  /* terrain restriction string    */   trainterrain,
  /* num items in cargo list       */   5,
  {V_MILIT, V_SHELL, V_GUN, V_FOOD, V_PETROL },
  {    200,    2000,   500,    100,      300 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TRAIN,
  /* lcms & hcms to build          */   40, 40,
  /* armor & speed ratings         */   10, 95,
  /* visibility & viewing range    */   20, 1,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "express",
  /* tech & cost to build          */   30, 4500,
  /* special features              */   0,
  /* mpg rating                    */   mpg(80),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   trainterrain,
  /* num items in cargo list       */   5,
  { V_CIVIL, V_MILIT, V_FOOD, V_UW, V_PETROL },
  {     500,      40,    200,  200,      100 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TANK,
  /* lcms & hcms to build          */	20, 30,
  /* armor & speed ratings         */	80, 80,
  /* visibility & viewing range    */	3, 3,
  /* firing range, number of guns  */	8, 3,
  /* ship name                     */	"hellraiser",
  /* tech & cost to build          */	150, 3100,
  /* special features              */	M_MSL,
  /* mpg rating                    */   40,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 2,
  /* terrain restriction string    */   tankterrain,
  /* num items in cargo list       */	5,
  { V_MILIT, V_FOOD, V_SHELL, V_GUN, V_PETROL },
  {       3,     25,      70,     5,       10}},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   40, 15,
  /* armor & speed ratings         */   30, 70,
  /* visibility & viewing range    */   20, 3,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "hydrofoil",
  /* tech & cost to build          */   165, 1000,
  /* special features              */   0,
  /* mpg rating                    */   mpg(25),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   6,
  { V_MILIT, V_CIVIL, V_UW, V_SHELL, V_FOOD, V_PETROL }, 
  {       5,      25,   40,      20,     50,       10 }},
  
/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TANK,
  /* lcms & hcms to build          */	20, 10,
  /* armor & speed ratings         */	45, 60,
  /* visibility & viewing range    */	3, 2,
  /* firing range, number of guns  */	3, 4,
  /* ship name                     */	"scorpion",
  /* tech & cost to build          */	120, 1200,
  /* special features              */	M_MSL,
  /* mpg rating                    */   45,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 1,
  /* terrain restriction string    */   tankterrain,
  /* num items in cargo list       */	5,
  { V_MILIT, V_FOOD, V_SHELL, V_GUN, V_PETROL },
  {       2,     20,      30,     5,       10}},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_TANK,
  /* lcms & hcms to build          */	8, 2,
  /* armor & speed ratings         */	1, 60,
  /* visibility & viewing range    */	2, 1,
  /* firing range, number of guns  */	2, 1,
  /* ship name                     */	"stinger",
  /* tech & cost to build          */	80, 100,
  /* special features              */	M_ATV,
  /* mpg rating                    */   100,
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   atvterrain,
  /* num items in cargo list       */	5,
  { V_MILIT, V_FOOD, V_SHELL, V_GUN, V_PETROL },
  {       2,     10,      10,     1,        5}},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   50, 50,
  /* armor & speed ratings         */   20, 10,
  /* visibility & viewing range    */   80, 7,
  /* firing range, number of guns  */   1, 1,
  /* ship name                     */   "asw platform",
  /* tech & cost to build          */   180, 1700,
  /* special features              */   M_HELI|M_FOOD|M_SONAR|M_SPY|M_MSL,
  /* mpg rating                    */   mpg(30),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 5,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_FOOD, V_GUN, V_SHELL, V_PETROL },
  {      20,    100,     1,     100,      100 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   20, 10,
  /* armor & speed ratings         */   15, 20,
  /* visibility & viewing range    */   4, 3,
  /* firing range, number of guns  */   2, 1,
  /* ship name                     */   "minisub",
  /* tech & cost to build          */   80, 1000,
  /* special features              */   M_SUB|M_TORP|M_MINE|M_SONAR,
  /* mpg rating                    */   mpg(100),
  /* max air (submarines only)     */	1000,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_SHELL, V_GUN, V_FOOD, V_PETROL },
  {       5,       8,     2,     20,        5 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   80, 60,
  /* armor & speed ratings         */   5, 25,
  /* visibility & viewing range    */   35, 3,
  /* firing range, number of guns  */   0, 0,
  /* ship name                     */   "ocean liner",
  /* tech & cost to build          */   100, 5000,
  /* special features              */   0,
  /* mpg rating                    */   mpg(50),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   0, 0,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   4,
  { V_CIVIL, V_MILIT, V_FOOD, V_PETROL },
  {     900,      50,    400,       40 }},

/* --------------------------------------------------------------------------- */

{ /* vehicle classification        */   CLASS_SHIP,
  /* lcms & hcms to build          */   50, 50,
  /* armor & speed ratings         */   70, 25,
  /* visibility & viewing range    */   35, 3,
  /* firing range, number of guns  */   4, 2,
  /* ship name                     */   "munitions ship",
  /* tech & cost to build          */   10, 950,
  /* special features              */   M_TAC|M_MSL,
  /* mpg rating                    */   mpg(50),
  /* max air (submarines only)     */	0,
  /* max tanks, planes             */   2, 2,
  /* terrain restriction string    */   shipterrain,
  /* num items in cargo list       */   5,
  { V_MILIT, V_SHELL, V_GUN, V_FOOD, V_PETROL },
  {      50,    1000,   300,     80,       50 }},

/* --------------------------------------------------------------------------- */

{ 0,0,0,0,0,0,0,0,0,"",0,0,0,0,0,0,0,{0},{0}, }

};

int     shp_maxno = (sizeof(mchr) / sizeof(struct mchrstr)) - 1;

#ifdef TRADECENTERS
struct classstr class[] = {
/*    class name      	build	load/unload	stop */
    {"ship",    	"h",  	"hx", 		"~"  }, /* 0 CLASS_SHIP    */
    {"tank",    	"z",  	"~=", 		"~"  }, /* 1 CLASS_TANK    */
    {"balloon", 	"h*", 	"h*x",		"~"  }, /* 2 CLASS_BALLOON */
    {"train",   	"z",  	"~=", 		"~"  }, /* 3 CLASS_TRAIN   */
    {"",        	"",   	"",   		""   }
    };
#else
struct classstr class[] = {
/*    class name      	build	load/unload	stop */
    {"ship",    	"h",  	"h", 		"~"  }, /* 0 CLASS_SHIP    */
    {"tank",    	"z",  	"~=", 		"~"  }, /* 1 CLASS_TANK    */
    {"balloon", 	"h*", 	"h*",		"~"  }, /* 2 CLASS_BALLOON */
    {"train",   	"z",  	"~=", 		"~"  }, /* 3 CLASS_TRAIN   */
    {"",        	"",   	"",   		""   }
    };
#endif

