#ifndef lint
static char *RCSid = "$Header: bmap.c,v 1.1 89/12/14 08:20:08 jay Exp $";
#endif /* not lint */

/*
 * bmap.c
 *
 * empire bigmap: probably the second most-used command in empire.
 *
 * Dave Pare, 1986
 * With hack by Jay Phillips and Bill Frolik, 1989
 *
 * 10/18/90 bf@hp - added 'save' and 'load' options
 * 6/25/90 bf@hp - let player modify own bigmap and show mined sectors
 */

#include <stdio.h>
#include <fcntl.h>
#include <dirent.h>
#include "gamesdef.h"
#include "misc.h"
#include "var.h"
#include "sect.h"
#include "xy.h"
#include "nsc.h"
#include "file.h"
#include "deity.h"
#include "nat.h"
#include "map.h"

#include "bigmap.h"

bmap()
{
	extern	int aborted;
	extern	char *argp[];
	register unsigned char *ptr;
	struct	nstr_sect ns;
	struct	natstr *np;
	struct	sctstr sect;
	struct	range range;
	int	i,showmines;
	char	wmap[WORLD_Y][MAPWIDTH(1)];
	char	newdes,temp;
	coord	y;

	if (!strcmp(argp[1],"list")) {
	    DIR *dirp;
	    char *p,*fspec;
	    struct dirent *dp;
	    if (!(p=argp[2])) p = ".";
	    fspec = "*";
	    dirp = opendir(p);
	    if (!readdir(dirp)) {
		fspec = p;
		while (*fspec) fspec++;
		while (fspec>p && *fspec!='/') fspec--;
		*fspec++ = '\0';
	    }
	    closedir(dirp);
	    dirp = opendir(p);
	    while ((dp = readdir(dirp)) != NULL)
		bmload(dp->d_name,0,1,&ns);
	    closedir(dirp);
	    return RET_OK;
	}

	if (!strcmp(argp[1],"delete")) {
	    return(bmdelete(argp[2]));
	}

	if (!strcmp(argp[1],"load")) {
	    return(bmload(argp[2],argp[3],0,&ns));
	}

	np = getnatp(cnum);
	if (!snxtsct(&ns, argp[1])) return RET_FAIL;
	ns.ncond = 0; /* zap any conditionals */
	xyrelrange(np, &ns.range, &range);

/*	Changed to fix a bug in BMAP, by  Frederic Chauveau <fmc@cnam.cnam.fr>
            Dec 26, 1991 
           if (!strcmp(argp[2],"save")) */
	if (argp[2] && !strcmp(argp[2],"save"))
	    return(bmsave(argp[3],&ns));

	/*
	**  Allow user to bruteforce change his bmap database
	**  using the syntax "bmap <sector(s)> <newchar> [!!!]".
	**  If the '!!!' third parameter is present, perform the
	**  command with no questions asked; otherwise, give
	**  the guy a chance to abort the change.
	*/

	newdes = (argp[2] && *argp[2]) ? *argp[2] : 0;
	if (showmines = (newdes==BMAP_MINE_CHAR)) newdes=0;

	if (newdes)
	    if (!oktomodify(argp[3])) return RET_FAIL;

	if (!newdes) border(&range, "     ", "");
	blankfill((char *)wmap, &ns.range, 1);
	while (bmnxtsct(&ns) && !aborted) {
	    if (newdes == BMAP_MINE_TOGGLE) 
		setbigmap(ns.x,ns.y,0x82);
	    else if (newdes) 
		setbigmap(ns.x,ns.y,newdes);
	    else {
		ptr = (unsigned char *)&wmap[ns.dy][ns.dx];
		if ((*ptr=(unsigned char)(getbigmap(ns.x, ns.y))) & 0x80)
		    *ptr = showmines ? BMAP_MINE_CHAR : (*ptr & 0x7F);
	    }
	}
	if (newdes) {
	    writebigmap();
	    return RET_OK;
	}
	if (aborted)
		return RET_OK;
	for (y=ns.range.ly, i=0; i < ns.range.height; y++, i++) {
		int yval;

		yval = yrel(np, y);
		pr(fmt("%4d %s %-4d\n", yval, wmap[i], yval));
		if (y >= WORLD_Y)
			y -= WORLD_Y;
	}
	border(&range, "     ", "");
	return RET_OK;
}

oktomodify(p)
char *p;
{
    if (!p || strcmp(p,"!!!")) {
	p = getstarg("","Are you SURE you want to modify the map? (n) ");
	if (!p || (*p != 'y')) return 0;
	p = getstarg("", "Really REALLY sure? (n) ");
	if (!p || (*p != 'y')) return 0;
    }
    return 1;
}

bmsave(name,np) /* save region as a bigmap file */
char *name;
register struct nstr_sect *np;
{
    extern char bigmap[];
    FILE *bmfile;
    int x,y,retcode,i,maker;
    long chksum;
    unsigned char *bmbuf,*s;
    maker = cnum;
    retcode = RET_FAIL;
    s = bmbuf = (unsigned char *)malloc(WORLDSIZE+50);
    if (!bmbuf || !(bmfile=fopen(name,"wb"))) {
	pr(fmt("Can't create '%s'\n",name));
    } else {
	fwrite("BMAP",1,4,bmfile);
	fwrite(&maker,sizeof(maker),1,bmfile);
	fwrite(&np->x,sizeof(np->x),1,bmfile);
	fwrite(&np->y,sizeof(np->y),1,bmfile);	
	fwrite(&np->cx,sizeof(np->cx),1,bmfile);
	fwrite(&np->cy,sizeof(np->cy),1,bmfile);
	fwrite(&np->dx,sizeof(np->dx),1,bmfile);
	fwrite(&np->dy,sizeof(np->dy),1,bmfile);
	fwrite(&np->type,sizeof(np->type),1,bmfile);
	fwrite(&np->range.lx,sizeof(np->range.lx),1,bmfile);
	fwrite(&np->range.width,sizeof(np->range.width),1,bmfile);
	fwrite(&np->range.height,sizeof(np->range.height),1,bmfile);
	chksum  = (long)maker;
	chksum += (long)np->x;
	chksum += (long)np->y;
	chksum += (long)np->cx;
	chksum += (long)np->cy;
	chksum += (long)np->dx;
	chksum += (long)np->dy;
	chksum += (long)np->type;
	chksum += (long)np->range.lx;
	chksum += (long)np->range.width;
	chksum += (long)np->range.height;
	while (bmnxtsct(np)) {
	    x  = normalize(np->x,WORLD_X);
	    y  = normalize(np->y,WORLD_Y);
	    if (valid(x,y)) {
		*s = stype(x,y);
		chksum += (long)(*s);
		s++;
	    }
	}
	fwrite(bmbuf,1,s-bmbuf,bmfile);
	pr(fmt("%d sector(s) saved in '%s'\n",s-bmbuf,name));
	fwrite(&chksum,sizeof(chksum),1,bmfile);
	fclose(bmfile);
	retcode = RET_OK;
    }
    free(bmbuf);
    return retcode;
}

bmdelete(name) /* delete a bigmap file */
char *name;
{
    FILE *bmfile;
    unsigned char bmbuf[5];
    if ((bmfile = fopen(name,"rb")) <= 0) {
	pr(fmt("Can't find '%s'\n",name));
    } else {
	fread(bmbuf,1,4,bmfile);
	fclose(bmfile);
	if (strncmp(bmbuf,"BMAP",4)) {
	    pr(fmt("'%s' is not a bmap savefile\n",name));
	} else {
	    unlink(name);
	    pr(fmt("'%s' deleted\n",name));
	}
    }
}

bmload(name,doit,listonly,np) /* load a region from a bigmap file */
char *name,*doit;
int listonly;
register struct nstr_sect *np;
{
    extern char bigmap[];
    FILE *bmfile;
    int i,n,x,y,retcode,maker;
    coord ulx,uly;
    long chksum,check;
    unsigned char *bmsav,old,bmbuf[5],*s;
    retcode = RET_FAIL;
    bmsav = (unsigned char *)malloc(WORLDSIZE);
    for (i=0;i<WORLDSIZE;i++) bmsav[i] = bigmap[i];
    if (!bmsav || ((bmfile=fopen(name,"rb")) <= 0)) {
	pr(fmt("Can't open '%s'\n",name));
    } else {
	fread(bmbuf,1,4,bmfile); 
	if (strncmp(bmbuf,"BMAP",4)) goto bmfail;
	fread(&maker,sizeof(maker),1,bmfile);
	fread(&np->x,sizeof(np->x),1,bmfile); 
	fread(&np->y,sizeof(np->y),1,bmfile);
	fread(&np->cx,sizeof(np->cx),1,bmfile);
	fread(&np->cy,sizeof(np->cy),1,bmfile);
	fread(&np->dx,sizeof(np->dx),1,bmfile);
	fread(&np->dy,sizeof(np->dy),1,bmfile);
	fread(&np->type,sizeof(np->type),1,bmfile);
	fread(&np->range.lx,sizeof(np->range.lx),1,bmfile);
	fread(&np->range.width,sizeof(np->range.width),1,bmfile);
	fread(&np->range.height,sizeof(np->range.height),1,bmfile);
	chksum  = (long)maker;
	chksum += (long)np->x;
	chksum += (long)np->y;
	chksum += (long)np->cx;
	chksum += (long)np->cy;
	chksum += (long)np->dx;
	chksum += (long)np->dy;
	chksum += (long)np->type;
	chksum += (long)np->range.lx;
	chksum += (long)np->range.width;
	chksum += (long)np->range.height;
	s = xyas(np->range.lx,np->y,cnum);
	ulx = atoi(s);
	while (*s && *s!=',') s++; s++;
	uly = atoi(s);
	if (np->range.width>1 || np->range.height>1)
	    pr(fmt("'%s' is the %dx%d region %d:%d,%d:%d",name,
		   np->range.width,np->range.height,
		   ulx,ulx+np->range.width-1,
		   uly,uly+np->range.height-1));
	else 
	    pr(fmt("'%s' is the single sector at %s",name,
		   xyas(np->range.lx,np->y,cnum)));
	pr(fmt(" (%s map)\n",cname(maker)));
	if (listonly) goto bmnoload;
	if (!oktomodify(doit)) goto bmabort;
	for (i=n=0;bmnxtsct(np);) {
	    x  = normalize(np->x,WORLD_X);
	    y  = normalize(np->y,WORLD_Y);
	    if (valid(x,y)) {
		if (fread(bmbuf,1,1,bmfile) != 1) goto bmfail;
		chksum += (long)((unsigned char)(*bmbuf));
		old = stype(x,y);
		if ((old==' ') ||
		    ((*bmbuf!=' ') && (old!=*bmbuf) &&
		     (old=='.' || old=='-' || old=='^'))) {
		    stype(x,y) = *bmbuf;
		    n++;
		}
		i++;
	    }
	}
	fread(&check,sizeof(check),1,bmfile);
	if (check == chksum) { 
	    pr(fmt("%d sector(s) read, %d sector(s) modified\n",i,n));
	    retcode = RET_OK;
	} else {
	bmfail:	
	    if (!listonly) pr(fmt("'%s' is not a bmap savefile\n",name));
	bmabort:
	    if (!listonly) pr("bmap load aborted\n");
	bmnoload:
	    for (n=0;n<WORLDSIZE;n++) bigmap[n] = bmsav[n];
	}
	writebigmap();
	fclose(bmfile);
    }
    free(bmsav);
    return retcode;
}

int bmnxtsct(np) /* get the next sector in the range */
register struct nstr_sect *np;
{
    while (1) {
	np->dx++;
	np->x++;
	if (np->x >= WORLD_X)
	    np->x = 0;
	if (np->dx >= np->range.width) {
	    np->dx = 0;
	    np->x = np->range.lx;
	    np->dy++;
	    if (np->dy >= np->range.height)
		return 0;
	    np->y++;
	    if (np->y >= WORLD_Y)
		np->y = 0;
	}
	if ((np->y + np->x) & 01)
	    continue;
	if (np->type == NS_DIST) {
	    np->curdist = mapdist(np->x, np->y, np->cx, np->cy);
	    if (np->curdist > np->dist)
		continue;
	}
	np->id = sctoff(np->x, np->y);
	return 1;
    }
}
