#ifndef lint
static char *RCSid = "$Header: coas.c,v 1.1 89/12/14 08:19:27 jay Exp $";
#endif /* not lint */

/*
 * coas.c
 *
 * look at all ships in world.
 *
 * Keith Muller, 1983
 * and Dave Pare, 1986
 */

#include "misc.h"
#include "var.h"
#include "sect.h"
#include "ship.h"
#include "nat.h"
#include "xy.h"
#include "nsc.h"
#include "file.h"

#define	TSIZE	200

struct coast {
	struct coast *c_next;
	int c_spotted;
	int c_number;
	struct shpstr c_shp;
};

static	showship();

/*
 * format: coastwatch [<SECTS>]
 */
coas()
{
	extern char *argp[];
	struct sctstr sect;
	struct nstr_sect nstr;
	struct coast *cp;
	struct coast *list[TSIZE];
	int i, k, j, n;
	int vrange, srange, see;
	int x, y;
	int mink, minj, maxk, maxj;
	int nship = 0;
	float tech;
	struct	nstr_item ni;

	if (snxtsct(&nstr, argp[1]) == 0)
		return RET_SYN;
	for (i = 0; i < TSIZE; i++)
		list[i] = 0;
	cp = (struct coast *)malloc(sizeof(*cp));
	snxtitem_all(&ni, EF_SHIP);
	while (nxtitem(&ni, (char *) &cp->c_shp)) {

	    /* don't bother with stuff that's ours or POGO's */
	    if (cp->c_shp.shp_own == 0 || cp->c_shp.shp_own == cnum)
		continue;

	    /* don't bother with things loaded aboard ships */
	    if (cp->c_shp.shp_aboard >= 0) continue;

	    /* don't bother putting submerged subs in the table... */
	    if (issub(&(cp->c_shp)) && cp->c_shp.shp_depth>0) continue;

	    n = scthash(cp->c_shp.shp_x, cp->c_shp.shp_y, TSIZE);
	    cp->c_spotted = 0;
	    cp->c_number = i;
	    cp->c_next = list[n];
	    list[n] = cp;
	    cp = (struct coast *)malloc(sizeof(*cp));
	    nship++;
	}
	/* get that last one! */
	free((char *)cp);
	pr(fmt("- = [ Coastwatch report for %s ] = -\n", cname(cnum)));
	pr(fmt("  Country            Ship          Location\n"));
	tech = tfact(cnum, 1.0);
	while (nxtsct(&nstr, &sect) && nship) {
		if (sect.sct_own != cnum)
			continue;
#ifdef COASTWATCHRADAR
		see = (sect.sct_type==SCT_RADAR) ? 14 : 4;
#else
		see = 4;
#endif
		vrange = (int) (sect.sct_effic / 100.0 * see * tech);
		if (vrange < 1)
			vrange = 1;
		maxk = vrange;
		maxj = vrange * 2;
		vrange *= vrange;
		mink = -maxk;
		minj = -maxj;
		for (j = minj; j <= maxj && nship; j++) {
			x = xnorm(sect.sct_x + j);
			for (k = mink; k <= maxk && nship; k++) {
				if ((j + k) & 01)
					 continue;
				/* quick range check to save time... */
				srange = (j*j + 3*k*k)/4;
				if (vrange < srange) continue;
				y = ynorm(sect.sct_y + k);
				n = scthash(x, y, TSIZE);
				if (list[n] == 0)
					continue;
				nship -= showship(&list[n], x, y, srange);
			}
		}
	}
	/* free up the coast structs calloc'ed above */
	for (i = 0; i < TSIZE; i++) {
		while (cp = list[i]) {
			list[i] = cp->c_next;
			free((char *)cp);
		}
	}
	return RET_OK;
}

static showship(cpp, x, y, srange)
    struct coast **cpp;
    int	x;
    int	y;
    int srange; /* ship distance squared */
{
    register struct coast *cp;
    register struct coast *todelete = 0;
    register struct coast **prev;
    register struct mchrstr *mcp;
    char	*name;
    int	nship = 0;
    int mass;
    
    prev = 0;
    cp = *cpp;
    prev = cpp;
    do {
	/* we delete it, we free it. */
	if (todelete) {
	    free((char *)todelete);
	    todelete = 0;
	}
	if (cp->c_shp.shp_x != x || cp->c_shp.shp_y != y) {
	    prev = &(*prev)->c_next;
	    continue;
	}
	mcp = &mchr[cp->c_shp.shp_type];
	if (mcp->m_flags & M_FOOD)
	    name = "fishing vessel";
	else
	    name = mcp->m_name;
	mass = mcp->m_lcm + mcp->m_hcm*2;
	if (srange < 5) { /* 0-2 sectors away */
	    pr(fmt(" %12.12s (#%3d) %s #%d%s%s%s @ %s\n",
		   cname(cp->c_shp.shp_own),
		   cp->c_shp.shp_own,
		   name, cp->c_shp.shp_uid,
		   *(cp->c_shp.shp_name) ? " (" : "",
		   cp->c_shp.shp_name,
		   *(cp->c_shp.shp_name) ? ")" : "",
		   xyas(x, y, cnum)));
	} else if (srange < 17) { /* 3-4 sectors away */
	    pr(fmt(" %19.19s %s @ %s\n",
		   "???",
		   name, xyas(x, y, cnum)));
	} else if (mcp->m_visib >= 20) {
	    pr(fmt(" %19.19s (%svessel) @ %s\n",
		   "???",
		   (mass >= 120)
		   ? "large "
		   : ((mass >= 80) 
		      ? "medium "
		      : "small "),
		   xyas(x, y, cnum)));
	}
	*prev = cp->c_next;
	todelete = cp;
	nship++;
    } while (cp = cp->c_next);
    /* check that last one! */
    if (todelete)
	free((char *)todelete);
    return (nship);
}
