#ifndef lint
static char *RCSid = "$Header: expl.c,v 1.1 89/12/14 08:19:47 jay Exp $";
#endif

/*
** expl.c
**
** takeover unoccupied sectors
**
** Jeff Wallace, 1989
**
** 4/15/91 bf@hp - added minimum requirements, mob multipliers, and
**		   expedition cost per sector.
*/

#include "misc.h"
#include "var.h"
#include "sect.h"
#include "xy.h"
#include "nsc.h"
#include "item.h"
#include "file.h"
#include "deity.h"
#include "nat.h"

extern int civexplore;		/* minimum civ needed to 'explore' */
extern double civexploremob;	/* minimum mobility used by civ explore */
extern int milexplore;		/* minimum mil needed to 'explore' */
extern double milexploremob;	/* minimum mobility used by mil explore */

static	int explore_map();

expl()
{
	extern char *argp[];
	register int amount;
	struct sctstr sect,endsect,start;
	int mcost,infected,vtype,amt_src,amt_dst;
	int work,loyal,own,justtook,i;
	double weight,minmob;
	struct ichrstr *ip;
	coord x, y;
	char *p;

	if ((ip = whatitem(argp[1], "explore with what? (civ/mil) ")) == 0)
		return RET_SYN;
	vtype = ip->i_vtype;
	if ((vtype != V_CIVIL) && (vtype != V_MILIT)) {
		pr("You can only explore with civs and mil.\n");
		return RET_FAIL;
	}
	if ((p = getstarg(argp[2], "from sector : ")) == 0)
		return RET_SYN;
	if (!sarg_xy(p, &x, &y))
		return RET_SYN;
	if (!getsect(x, y, &sect) || !owner) {
		pr("Not yours\n");
		return RET_FAIL;
	}
	infected = getvar(V_PTIME, (char *)&sect, EF_SECTOR) == PLG_INFECT;
	if ((amt_src = getvar(vtype, (char *)&sect, EF_SECTOR)) <= 0) {
		pr(fmt("No %s in %s\n", ip->i_name, 
			xyas(sect.sct_x, sect.sct_y, cnum)));
		return RET_SYN;
	}
	own = sect.sct_own;
	if (vtype == V_CIVIL && sect.sct_oldown != own) {
		pr("You can't explore with conquered populace!\n");
		return RET_SYN;
	}
	if (vtype == V_CIVIL) {
	    work = sect.sct_work;
	    if (work != 100) pr("Warning: civil unrest\n");
	    loyal = sect.sct_loyal;
	    minmob = civexploremob;
	} else {
	    work = 100;
	    loyal = 0;
	    minmob = milexploremob;
	}
	
	amount = onearg(argp[3], fmt("Quantity? (max %d) ", amt_src));

	if (amount < ((vtype==V_CIVIL)?civexplore:milexplore)) {
	    pr(fmt("Exploration party requires at least %d civ or %d mil.\n",
		   civexplore,milexplore));
	    amount = 0;
	}

	if (amount > amt_src) {
	    amount = amt_src;
	    pr(fmt("Only exploring with %d.\n", amount));
	}

	if (amount <= 0)
	    return RET_SYN;
	
	weight = amount * ip->i_lbs;

	/*
	** Now parse the path and return ending sector.
	*/

	mcost = move_ground((char *)ip, &sect, &endsect,
		(double)sect.sct_mobil, weight, argp[4], 
		explore_map, &minmob);

	if (mcost < 0) {
	    pr("Explore aborted\n");
	    return RET_FAIL;
	}

	/*
	 * Re-read dest sector to minimize our
	 * window of vulnerability.  If we got this far, always
	 * charge the player mobility.
	 */
	getsect(endsect.sct_x, endsect.sct_y, &sect);
		/* charge the player mobility anyway 
		 * this is need to prevent users from making
		 * explore into a new 'test' command -- jeffw
		 */
/* this is commented out because players cannot currently occupy a
 * non-contiguous area of land with both civs and mil - Jim

	if (sect.sct_own == own) {
		pr("You can't explore a sector that you already own!\n");
		getsect(x, y, &start);
		start.sct_mobil -= mcost;
		putsect(&start);
		return RET_OK;
	}

*/

	if (sect.sct_x == x && sect.sct_y == y) {
	    /* didn't go anywhere? */
	    if (mcost > 0) pr("Welcome home.\n");
	    sect.sct_mobil -= mcost;
	    putsect(&sect);
	    return RET_OK;
	}

	justtook = 0;
	if (!god && !owner && sect.sct_type != SCT_SANCT) {
		(void) takeover(&sect, 0);
		justtook = 1;
		sect.sct_oldown = own;
		sect.sct_work = work;
		sect.sct_loyal = loyal;
	}

	if (vtype == V_CIVIL && sect.sct_oldown != cnum) {
		pr("Your civilians don't want to stay!\n");
		amount = 0;
	}
	amt_dst = getvar(vtype, (char *)&sect, EF_SECTOR);
	if (32767 - amt_dst < amount) {
		amount = 32767 - amt_dst;
		pr(fmt("Only %d can be left there.\n", amount));
	}
	if (putvar(vtype, amount + amt_dst, (char *)&sect, EF_SECTOR) < 0) {
		pr(fmt("No more room in %s.\n", 
			xyas(sect.sct_x, sect.sct_y, cnum)));
		/* charge the player mobility anyway */
		amount = 0;
	}
	/*
	 * First remove commodities from source sector,
	 * and decrement mobility appropriately.
	 */
	getsect(x, y, &start);
	amt_src = getvar(vtype, (char *)&start, EF_SECTOR);
	amt_src -= amount;
	if (amt_src < 0) {
		pr(fmt("%s in %s are gone!\n", ip->i_name, 
			xyas(start.sct_x, start.sct_y, cnum)));
		amount = 0;
	}
	start.sct_mobil -= mcost;
	putvar(vtype, amt_src, (char *)&start, EF_SECTOR);
	putsect(&start);
	/*
	 * Now add commodities to destination sector,
	 * along with plague that came along for the ride.
	 * Takeover unowned sectors if not deity.
	 */
	if (!god && sect.sct_type != SCT_SANCT && justtook) {
		pr(fmt("Sector %s is now yours.\n", 
			xyas(sect.sct_x, sect.sct_y, cnum)));
		sect.sct_mobil = 0;
	}
	if (infected && getvar(V_PTIME, (char *)&sect, EF_SECTOR) == 0)
		putvar(V_PTIME, PLG_EXPOSED, (char *)&sect, EF_SECTOR);
	if (vtype == V_CIVIL) {
		/* It only takes one bad apple... */
		if (sect.sct_loyal < loyal)
			sect.sct_loyal = loyal;
		if (sect.sct_work > work)
			sect.sct_work = work;
	}
	putsect(&sect);
	return RET_OK;
}

/*ARGSUSED*/
static
explore_map(what, curx, cury, arg)
	char	*what;
	coord	curx, cury;
	char	*arg;
{
	struct	nstr_sect ns;
	struct	natstr *np;
	struct	sctstr sect;
	coord	rel_x, rel_y;
	char	*range;
	char	view[7];
	int	i;

	np = getnatp(cnum);
	rel_x = xrel(np, curx);
	rel_y = yrel(np, cury);
	range = fmt("%d:%d,%d:%d", rel_x-2, rel_x+2, rel_y-1, rel_y+1);
	if (!snxtsct(&ns, range))
		return RET_FAIL;
	i = 0;
	while (i < 7 && nxtsct(&ns, &sect)) {
		view[i] = dchr[sect.sct_type].d_mnem;
		switch (sect.sct_type) {
		case SCT_WATER:
		case SCT_RURAL:
		case SCT_MOUNT:
		case SCT_WASTE:
			break;
		default:
			if (sect.sct_own != cnum && !god)
				view[i] = '?';
			break;
		}
		setbigmap(ns.x, ns.y, view[i]);
		i++;
	}
	writebigmap();
	if (!getsect(curx, cury, &sect))
		return RET_FAIL;
	pr(fmt("    %c %c         min gold fert  oil uran\n", view[0], view[1]));
	pr(fmt("   %c %c %c        %3d %4d %4d %4d %4d\n",
		view[2], view[3], view[4],
		sect.sct_min, sect.sct_gmin, sect.sct_fertil, sect.sct_oil,
		sect.sct_uran));
	pr(fmt("    %c %c\n", view[5], view[6]));
	return RET_OK;
}
