#ifndef lint
static char *RCSid = "$Header: map.c,v 1.1 89/12/14 08:20:08 jay Exp $";
#endif /* not lint */

/*
** map.c
**
** empire map: probably the most-used command in empire.
**
** Dave Pare, 1986
**
** bf@hp 6/5/91 -
**
**	Expanded syntax for POGO to highlight certain
**	sectors:
**		map [sects] [country] [thing]
**	where
**		[country] is a country name or number, or '*'
**		[thing] is one of the following:
**	
**			tank	train	balloon	
**			plane	ship	land
**	
*/

#include "misc.h"
#include "var.h"
#include "sect.h"
#include "xy.h"
#include "nsc.h"
#include "file.h"
#include "deity.h"
#include "nat.h"
#include "map.h"

#include "ship.h"
#include "plane.h"
#define CLASS_LAND 98
#define CLASS_PLANE 99

map()
{
    extern int aborted;
    extern char *argp[];
    register char *ptr;
    struct nstr_sect ns;
    struct natstr *np;
    struct sctstr sect;
    struct range range;
    int	i;
    u_char bitmap[WORLD_X*WORLD_Y/8];
    char wmap[WORLD_Y][MAPWIDTH(1)];
    coord y;

    char *p;
    natid hilite;
    int hilitewhat;
    u_char hilitemap[WORLD_X*WORLD_Y/8];
    struct nstr_item ni;
    struct shpstr ship;
    struct plnstr plane;

    if (!snxtsct(&ns, argp[1]))	return RET_FAIL;
    np = getnatp(cnum);
    hilite = 0;
    if (!god) {
	bzero((char *)bitmap, sizeof(bitmap));
	bitinit(&ns, bitmap);
    } else {
	bzero((char *)hilitemap,sizeof(hilitemap));
	if (argp[2] && *argp[2]) {
	    if (*argp[2]=='*') hilite = 9999;
	    else if ((hilite = natarg(argp[2],"")) <= 0) {
		pr("Bad country name/number\n");
		return RET_SYN;
	    }
	    p = getstarg(argp[3],
        "Locations of what (land, ship, tank, balloon, train, plane)? ");
	    if (!p || !*p) hilite = 0;
	    else switch (*p) {
	    case 'l': hilitewhat=CLASS_LAND; break;
	    case 's': hilitewhat=CLASS_SHIP; break;
	    case 'b': hilitewhat=CLASS_BALLOON; break;
	    case 't': hilitewhat=(p[1]=='r') ? CLASS_TRAIN : CLASS_TANK;break;
	    case 'p': hilitewhat=CLASS_PLANE; break;
	    default:  hilite = 0;
	    }
	    if (hilite) switch (hilitewhat) {
	    case CLASS_PLANE:
		if (snxtitem(&ni, EF_PLANE, "*"))
		    while (nxtitem(&ni, (char *)&plane)) {
			if (hilite==9999 || plane.pln_own == hilite)
			    setbit(plane.pln_x,plane.pln_y,hilitemap);
		    }
	    case CLASS_LAND:
		/* do nothing */
		break;
	    default:
		if (snxtitem(&ni, EF_SHIP, "*"))
		    while (nxtitem(&ni, (char *)&ship)) {
			if ((hilite==9999 || ship.shp_own==hilite) && 
			    mchr[ship.shp_type].m_class==hilitewhat)
			    setbit(ship.shp_x,ship.shp_y,hilitemap);
		    }
	    }
	}
    }
	
    /* zap any conditionals */
    ns.ncond = 0;
    xyrelrange(np, &ns.range, &range);
    border(&range, "     ", "");
    blankfill((char *)wmap, &ns.range, 1);
    while (nxtsct(&ns, &sect) && !aborted) {
	if (!god && !getbit(ns.x, ns.y, bitmap)) continue;
	ptr = &wmap[ns.dy][ns.dx];
	if (sect.sct_type > SCT_MAXDEF) *ptr = '?';
	else {
	    *ptr = dchr[sect.sct_type].d_mnem;
	    switch (sect.sct_type) {
	    case SCT_WATER:
	    case SCT_RURAL:
	    case SCT_MOUNT:
	    case SCT_WASTE:
		break;
	    default:
		if (sect.sct_own != cnum && !god)
		    *ptr = '?';
		break;
	    }
	    setbigmap(ns.x, ns.y, *ptr);
	    if (hilite)
		if (hilitewhat==CLASS_LAND && 
		    (hilite==9999 || hilite==sect.sct_own))
		    *ptr |= 0x80;
		else if (getbit(ns.x,ns.y,hilitemap)) 
		    *ptr |= 0x80;
	}
    }
    writebigmap();
    if (aborted) return RET_OK;
    for (y=ns.range.ly, i=0; i < ns.range.height; y++, i++) {
	int yval;
	yval = yrel(np, y);
	pr(fmt("%4d %s %-4d\n", yval, wmap[i], yval));
	if (y >= WORLD_Y)
	    y -= WORLD_Y;
    }
    border(&range, "     ", "");
    return RET_OK;
}
