#ifndef lint
static char *RCSid = "$Header: plane.c,v 1.1 89/12/14 13:30:49 jay Exp $";
#endif

/*
**  plane.c
**
**  Do "production" for planes -- make more efficient,
**  charge for military, etc.
**
**  Dave Pare, 1986
**
**  Updated by Bill Frolik, 1991
*/

#include "misc.h"
#include "var.h"
#include "sect.h"
#include "plane.h"
#include "path.h"
#include "ship.h"
#include "nat.h"
#include "file.h"

int prod_plane(etus)
int etus;
{
    extern double money_mil;
    extern double flight_pay;
    extern double money_plane;
    extern int plane_mob_max;
    extern int etu_per_update;
    extern int orbit_rate;
    extern long money[MAXNOC];
    extern long air_money[MAXNOC];
    register struct plnstr *pp;
    struct natstr *np;
    int	odir,i,n;
    struct shpstr *shp;
    struct mchrstr *mp;
    struct plchrstr *plp;
    struct sctstr *sp;
    int	delta;
    int	mult;
    int	eff;
    int	avail;
    int	w_p_eff;
    int	left;
    int	used;
    int start_money;

    for (n=0; pp = getplanep(n); n++) {

	if (pp->pln_own == 0) continue;
	if (pp->pln_effic < 10) {
	    pp->pln_own = 0;
	    continue;
	}
	if (pp->pln_flags & PLN_BOXED) continue;

	plp = &plchr[pp->pln_type];

	/* If plane is aboard something, make sanity checks */
	if (pp->pln_ship >= 0) {
	    if ((shp=getshipp(pp->pln_ship)) == 0) {
		if (--shp->shp_nplane < 0)
		    shp->shp_nplane = 0;
		pp->pln_own = 0;
		continue;
	    }
	    mp = &mchr[shp->shp_type];
	    if (!planefitsship(plp->pl_flags,mp->m_flags)) {
		if (--shp->shp_nplane < 0)
		    shp->shp_nplane = 0;
		pp->pln_own = 0;
		continue;
	    }
	}

	np = getnatp(pp->pln_own);
	start_money = np->nat_money;
	mult = (np->nat_level[NAT_TLEV] < pp->pln_tech * 0.85) ? 2 : 1;

	/* Pay the crew.  If the national tech level is less than 0.85
	   times the plane's tech level, the crew gets double pay. */

	np->nat_money += (int)(mult*etus*(plp->pl_crew*money_mil*flight_pay));

	/* If plane owner is broke, slowly reduce plane efficiency. */

	if (money[pp->pln_own] < 0) {
	    delta = random() % (etus/2);
	    eff = pp->pln_effic - delta;
	    if (eff < 0) eff = 0;
	    pp->pln_effic = eff;
	    if (pp->pln_effic < 10) {
		wu(0, pp->pln_own,
		   fmt("%s #%d lost to lack of maintenance\n",
		       plp->pl_name,n));
		pp->pln_own = 0;
	    }
	    air_money[pp->pln_own] += np->nat_money - start_money;
	    continue;
	}

	/* Pay for plane maintenance.  If the national tech level is
	   less than 0.85 times the plane's tech, upkeep costs twice as
	   much. */

	np->nat_money += (int)
	    (mult * etus * dmin(-1.0, plp->pl_cost * money_plane));
	air_money[pp->pln_own] += np->nat_money - start_money;
	start_money = np->nat_money;
	
	/* If we're messing with a satellite ... */
	if (pp->pln_flags & PLN_LAUNCHED) {
	    odir = pp->pln_flags & PLN_ORBITDIR;
	    if (odir>0 && odir<7) {
		i = 1 + (pp->pln_flags & PLN_ORBITVEL)>>3;
		i = (i * etu_per_update) / orbit_rate;
		pp->pln_x += (i * diroff[odir][0]);
		pp->pln_y += (i * diroff[odir][1]);
		pp->pln_x = xnorm(pp->pln_x);
		pp->pln_y = ynorm(pp->pln_y);
	    }
	    if (pp->pln_mobil == plane_mob_max) /* autoupdate bigmap */
		satmapnopr(pp->pln_x, pp->pln_y, (int)pp->pln_effic, 
			   (int)techfact(pp->pln_tech, 20.0),
			   plchr[pp->pln_type].pl_flags);
	    continue; /* that's all */
	}

	/* Increase plane efficiency.  This requires workforce and cash. */

	sp = getsectp(pp->pln_x, pp->pln_y);
	left = 100 - pp->pln_effic;
	if (left <= 0) continue;
	avail = sp->sct_avail * 100;
	w_p_eff = 20 + (plp->pl_lcm + 2 * plp->pl_hcm);
	delta = roundavg((double)avail/w_p_eff);
	if (delta <= 0)    continue;
	if (delta > etus)  delta = etus;
	if (delta > left)  delta = left;
	used = delta * w_p_eff;

	/* I didn't use roundavg here, because I want to penalize
	   the player with a large number of planes. */

	avail = (sp->sct_avail * 100 - used) / 100;
	if (avail < 0) avail = 0;
	sp->sct_avail = avail;
	if (sp->sct_type != SCT_AIRPT) delta /= 3;
	np->nat_money -= roundavg(mult*delta*plp->pl_cost/100.0);
	air_money[pp->pln_own] += np->nat_money - start_money;
	pp->pln_effic += delta;
    }
}

satmapnopr(cx, cy, eff, range, flags)
int cx,cy,eff,range,flags;
{
    char c;
    struct sctstr sect;
    struct nstr_sect ns;
    range = (int) (range * (eff / 100.0));
    snxtsct_dist(&ns, cx, cy, range);
    while (nxtsct(&ns, &sect)) {
	c = dchr[sect.sct_type].d_mnem;
	if (!(flags & P_I)) 
	    switch(sect.sct_type) {
	    case SCT_RURAL:
	    case SCT_WATER:
	    case SCT_WASTE:
	    case SCT_MOUNT: break;
	    default: if (sect.sct_own) c = '?';
	    }
	setbigmap(ns.x, ns.y, c);
    }
    writebigmap();
}


